{
    "Configuration": {
        "stopAt": {{ simulation_time }},
        "timeStepSize": {{ time_step }},
        "particleRadius": {{ particle_radius }},
        "simulationMethod": 4,
        "boundaryHandlingMethod": 0,
        "kernel": 1,
        "cflMethod": {{ adaptive_time_step | int }},
        "cflFactor": 0.5,
        "cflMinTimeStepSize": 0.0001,
        "cflMaxTimeStepSize": 0.005,
        "gravitation": [0, 0, -9.81],
        "gradKernel": 1,
        "enableVTKExport": true,
        "dataExportFPS": {{ data_export_rate }},
        "particleAttributes": "velocity;density",
        "enableZSort": {{ particle_sorting | tojson }}
    },
    "RigidBodies": [
        {
            "geometryFile": "{{ tank_filename }}",
            "translation": [0, 0, 0],
            "scale": {{ tank_dimensions }},
            "isDynamic": false
        }
    ],
    "Materials": [
        {
            "id": "Fluid",
            "density0": {{ fluid.density }},
            "viscosity": {{ fluid.kinematic_viscosity }},
            "viscosityMethod": 6
        }
    ],
    "FluidModels": [
        {
            "id": "Fluid",
            "particleFile": "{{ fluid_filename }}",
            "translation": {{ fluid_position }},
            "scale": {{ fluid_dimensions }},
            "initialVelocity": {{ fluid_velocity }}
        }
    ]
}