# coding: utf-8

# flake8: noqa
"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

__version__ = "1.0.0"

# import ApiClient
from inductiva.client.api_client import ApiClient

# import Configuration
from inductiva.client.configuration import Configuration

# import exceptions
from inductiva.client.exceptions import OpenApiException
from inductiva.client.exceptions import ApiAttributeError
from inductiva.client.exceptions import ApiTypeError
from inductiva.client.exceptions import ApiValueError
from inductiva.client.exceptions import ApiKeyError
from inductiva.client.exceptions import ApiException
